@@ -127,7 +127,9 @@ void SubDocument::parse(shared_ptr<WPSContentListener> &listener, libwps::SubDoc
 //! the state of WPS4
 struct State
 {
-	State() : m_isDosFile(false), m_eof(-1), m_fontType(libwps_tools_win::Font::UNKNOWN),
+	explicit State(libwps_tools_win::Font::Type fontType) :
+		m_isDosFile(false), m_eof(-1), m_fontType(fontType),
+		m_OEMFontType(libwps_tools_win::Font::UNKNOWN),
 		m_pageSpan(), m_noFirstPageHeader(false), m_noFirstPageFooter(false),
 		m_numColumns(1), m_actPage(0), m_numPages(0)
 	{
@@ -136,8 +138,10 @@ struct State
 	bool m_isDosFile;
 	//! the last file position
 	long m_eof;
-	//! the document code page
+	//! the user font type
 	libwps_tools_win::Font::Type m_fontType;
+	//! the OEM code page
+	libwps_tools_win::Font::Type m_OEMFontType;
 	//! the actual document size
 	WPSPageSpan m_pageSpan;
 	bool m_noFirstPageHeader /* true if the first page has no header */;
@@ -148,11 +152,12 @@ struct State
 }
 
 // constructor, destructor
-WPS4Parser::WPS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header) :
+WPS4Parser::WPS4Parser(RVNGInputStreamPtr &input, WPSHeaderPtr &header,
+                       libwps_tools_win::Font::Type encoding) :
 	WPSParser(input, header),
 	m_listener(), m_graphParser(), m_textParser(), m_state()
 {
-	m_state.reset(new WPS4ParserInternal::State);
+	m_state.reset(new WPS4ParserInternal::State(encoding));
 	m_graphParser.reset(new WPS4Graph(*this));
 	m_textParser.reset(new WPS4Text(*this, input));
 }
@@ -177,12 +182,19 @@ int WPS4Parser::numColumns() const
 	return m_state->m_numColumns;
 }
 
-libwps_tools_win::Font::Type WPS4Parser::getDocumentFontType() const
+libwps_tools_win::Font::Type WPS4Parser::getDefaultFontType() const
 {
-	return m_state->m_fontType;
+	if (m_state->m_fontType != libwps_tools_win::Font::UNKNOWN)
+		return m_state->m_fontType;
+	return version()<=2 ? libwps_tools_win::Font::DOS_850 : libwps_tools_win::Font::WIN3_WEUROPE;
 }
 
-bool WPS4Parser::getColor(int id, uint32_t &color) const
+libwps_tools_win::Font::Type WPS4Parser::getOEMFontType() const
+{
+	return m_state->m_OEMFontType;
+}
+
+bool WPS4Parser::getColor(int id, WPSColor &color) const
 {
 	if (m_state->m_isDosFile)
 	{
@@ -196,7 +208,7 @@ bool WPS4Parser::getColor(int id, uint32_t &color) const
 			WPS_DEBUG_MSG(("WPS4Parser::getColor(): unknown Dos color id: %d\n",id));
 			return false;
 		}
-		color=colorDosMap[id];
+		color=WPSColor(colorDosMap[id]);
 		return true;
 	}
 	static const uint32_t colorMap[]=
@@ -207,14 +219,14 @@ bool WPS4Parser::getColor(int id, uint32_t &color) const
 		0x0000FF, 0x00FFFF,
 		0x00FF00, 0xFF00FF,	0xFF0000, 0xFFFF00,
 		0x808080, 0xFFFFFF,	0x000080, 0x008080,
-		0x008000, 0x800080,	0x808000, 0xC0C0C0
+		0x008000, 0x800080,	0x800000, 0xC0C0C0
 	};
 	if (id < 0 || id >= 16)
 	{
 		WPS_DEBUG_MSG(("WPS4Parser::getColor(): unknown color id: %d\n",id));
 		return false;
 	}
-	color = colorMap[id];
+	color = WPSColor(colorMap[id]);
 	return true;
 }
 
@@ -310,9 +322,9 @@ int WPS4Parser::readObject(RVNGInputStreamPtr input, WPSEntry const &entry)
 	return id;
 }
 
-void WPS4Parser::sendObject(Vec2f const &size, int id)
+void WPS4Parser::sendObject(WPSPosition const &position, int id)
 {
-	return m_graphParser->sendObject(size, id);
+	return m_graphParser->sendObject(position, id);
 }
 
 void WPS4Parser::send(WPSEntry const &entry, libwps::SubDocumentType)
@@ -401,6 +413,7 @@ void WPS4Parser::parse(librevenge::RVNGTextInterface *documentInterface)
 		throw (libwps::ParseException());
 	}
 	m_listener->startDocument();
+	m_textParser->sendObjects(-1);
 	WPSEntry ent = m_textParser->getMainTextEntry();
 	if (ent.valid())
 		m_textParser->readText(ent);
@@ -538,7 +551,11 @@ bool WPS4Parser::checkHeader(WPSHeader *header, bool /*strict*/)
 	uint8_t secondOffset = libwps::readU8(input);
 	if (secondOffset != 0xFE || firstOffset > 7) return false;
 	if (header)
+	{
+		if (firstOffset==1)
+			header->setNeedEncoding(true);
 		header->setMajorVersion(firstOffset<4 ? 2 : firstOffset<6 ? 3 : 4);
+	}
 	return true;
 }
 
@@ -584,11 +601,23 @@ bool WPS4Parser::findZones()
 		break;
 	case 4:
 		worksVersion=3;
-		f << "vers=Win3,";
+		if (apCreator==0x4e27)
+		{
+			f << "vers=Win3.0,";
+			apCreator = 0;
+		}
+		else
+			f << "vers=Win3,";
 		break;
 	case 6:
 		worksVersion=4;
-		f << "vers=Win4,";
+		if (apCreator==0x5375)
+		{
+			f << "vers=Win4.0,";
+			apCreator = 0;
+		}
+		else
+			f << "vers=Win4,";
 		break;
 	default:
 		WPS_DEBUG_MSG(("WPS4Parser::findZones: can not read the version\n"));
@@ -614,8 +643,8 @@ bool WPS4Parser::findZones()
 		libwps_tools_win::Font::Type type=libwps_tools_win::Font::getTypeForOEM((oem>>4)&0x7ff);
 		if (type != libwps_tools_win::Font::UNKNOWN)
 		{
-			m_state->m_fontType = type;
-			f << "codePage=" << libwps_tools_win::Font::getTypeName(type) << ",";
+			m_state->m_OEMFontType = type;
+			f << "codePage=" << libwps_tools_win::Font::getTypeName(type).cstr() << ",";
 		}
 		else
 			f << "#codePage=" << std::hex << ((oem>>4)&0x7ff) << std::dec << ",";
@@ -960,7 +989,7 @@ bool WPS4Parser::readDocWindowsInfo(WPSEntry const &entry)
 	// Find in one file str[0]="C:\Databases\Elem 02-03.wdb",str[120]="Query1"
 	while (pos++ < 0x132)   // try to find some strings
 	{
-		char c = libwps::read8(input);
+		char c = char(libwps::read8(input));
 		if (c == '\0')
 		{
 			if (debSPos>=0)
@@ -996,9 +1025,9 @@ bool WPS4Parser::readDocWindowsInfo(WPSEntry const &entry)
 	if (dim[0] || dim[1])
 		f << "pageBorderDist=" << dim[1]/1440. << "x" << dim[0]/1440. << ",";
 	long val = libwps::readU8(input);
-	uint32_t color;
+	WPSColor color;
 	if (val && getColor(int(val), color))
-		f << "pageBorderColor=" << std::hex << color << std::dec << ",";
+		f << "pageBorderColor=" << color << ",";
 	else if (val)
 		f << "#pageBorderColor=" << std::hex << val << std::dec << ",";
 	val = libwps::readU8(input);
